using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;

using System;
using System.Collections;

using System.Xml;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Reflection;

namespace sbrcli
{
	public abstract class BookFormat
	{

        static string filters = null;
        public string ABBREVIATION = "BOOK";
        public string DESCRIPTION = "Book";
        public string COMMENTS = "Exported from sbrcli (" + GlobalMemory.AUTHOR_WEBSITE + ")";
        private Hashtable chapterCount = new Hashtable();
		private string m_filename;
		private static System.Xml.XmlDocument xmldoc = new System.Xml.XmlDocument();
		
		private bool process_complete = false;
		
        public static bool _internal_maincss = false;

        public static BookFormat book_format=null;
        public bool CLEANING = false;

        public string rtf_header = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f1\fnil\fprq2\fcharset161 TITUS Cyberbit Basic;}{\f2\fnil\fprq2\fcharset177 TITUS Cyberbit Basic;}{\f3\fnil\fprq2\fcharset0 TITUS Cyberbit Basic;}{\f4\fnil\fprq2\fcharset136 PMingLiU;}{\f5\fnil\fprq2\fcharset204 Georgia;}{\f6\fnil\fprq2\fcharset238 Georgia;}{\f7\fnil\fprq2\fcharset134 PMingLiU;}{\f8\fnil\fprq2\fcharset222 Cordia New;}{\f9\fnil\fprq2\fcharset129 Gulim;}{\f10\fnil\fprq2\fcharset178 TITUS Cyberbit Basic;}{\f11\fnil\fprq2\fcharset162 Georgia;}{\f12\fnil\fprq2\fcharset163 Times New Roman;}{\f13\fnil\fprq2\fcharset128 MS Mincho;}}{\colortbl ;\red0\green0\blue0;\red0\green0\blue255;\red0\green255\blue255;\red0\green255\blue0;\red255\green0\blue255;\red255\green0\blue0;\red255\green255\blue0;\red255\green255\blue255;\red0\green0\blue128;\red0\green128\blue128;\red0\green128\blue0;\red128\green0\blue128;\red128\green0\blue0;\red128\green128\blue0;\red128\green128\blue128;\red192\green192\blue192;}{\viewkind4\uc1\pard\plain\cf0\fs\par}";
        
		public BookFormat(string filename)
		{
			m_filename = filename;
			process_complete = false;
            _internal_maincss = false;
        }

        public static string getFormatTypes()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("\t0 - Auto Detect\r\n"); //idx = 1
            sb.Append("\t1 - XML Books (*.xml)\r\n"); //idx = 1
            sb.Append("\t2 - The Word Bible Books (*.gbk.twm)\r\n"); //idx = 2
            sb.Append("\t4 - e-Sword Bible 9.x Topics / References (*.topx *.refx)\r\n"); //idx = 4
            sb.Append("\t5 - e-Sword Bible HD References (*.refi)\r\n"); //idx = 5
            sb.Append("\t6 - SWORD Book Modules (*.zip)\r\n"); //idx = 6
            sb.Append("\t7 - Bible Analyzer Book (*.bk)\r\n"); //idx = 7
            sb.Append("\t8 - MySword for Android (*.bok.mybible.gz, *.bok.mybible)\r\n"); //idx = 8
            sb.Append("\t9 - STEP Files (BOOK.DAT)\r\n"); //idx = 9               
            sb.Append("\t10 - SwordSearcher/Forge for SwordSearcher (*.ss5book, *.txt)"); //idx = 10
            return sb.ToString();
        }

        public static BookFormat getInstance()
        {
            return book_format;
        }

        public static void setInstance(BookFormat mBookFormat)
        {
            book_format = mBookFormat;
        }

        public static BookFormat BookFactory(string filename)
        {
            book_format = null;
            if (filename.ToLower().EndsWith(".xml"))
            {                
                book_format = new XMLBookFormat(filename);            
            }
            else if (filename.ToLower().EndsWith(".gbk.twm"))
            {
                book_format = new TheWordBookFormat(filename);
            } 
            else if (filename.ToLower().EndsWith(".topx") || filename.ToLower().EndsWith(".refx"))
            {
                book_format = new TOPXBookFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".refi"))
            {
                book_format = new REFIBookFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".zip"))
            {
                book_format = new SwordBookFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".bk"))
            {
                book_format = new BibleAnalyzerBookFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".bok.mybible.gz"))
            {
                book_format = new MySword4AndroidBookFormat(filename, true);
            }
            else if (filename.ToLower().EndsWith(".bok.mybible"))
            {
                book_format = new MySword4AndroidBookFormat(filename, false);
            }
            else if (Path.GetFileName(filename).ToUpper().Equals("BOOK.DAT"))
            {
                book_format = new STEPBookFormat(filename);
            }            
            else if (filename.ToLower().EndsWith(".ss5book"))
            {
                book_format = new SwordSearcherBookFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".txt"))
            {
                StreamReader sr = File.OpenText(filename);
                string line = sr.ReadLine();
                sr.Close();
                if (line.StartsWith(";"))
                    book_format = new SwordSearcherBookFormat(filename);
                else
                {
                    // other books in text format
                }
            }
            else if (System.IO.Directory.Exists(filename))
            {
                //Its a directory
                if (System.IO.Directory.Exists(filename + Path.DirectorySeparatorChar+"mods.d"))
                {
                    book_format = new SwordBookFormat(filename);
                }                
            }
                              

            if (book_format != null)
            {
                if (cleaningRequired(book_format.FilterIndex))
                {
                    book_format.CLEANING = true;
                }

                if (book_format.ABBREVIATION == "BOOK")
                    book_format.ABBREVIATION = Path.GetFileNameWithoutExtension(filename);
                if (book_format.DESCRIPTION == "Book")
                    book_format.DESCRIPTION = Path.GetFileNameWithoutExtension(filename);
            }
            return book_format;
        }

        public static BookFormat BookFactory(string filename, int filterIdx)
        {
            if (filterIdx == 0)
                book_format = BookFactory(filename);
            else
                book_format = getBookFormat(filename, filterIdx);
            return book_format;
        }

        public static BookFormat getBookFormat(string filename, int filterIdx)
        {
            BookFormat book_format_export = null;
            switch (filterIdx)
            {
                case 1:
                    book_format_export = new XMLBookFormat(filename);
                    break;
                case 2:
                    book_format_export = new TheWordBookFormat(filename);
                    break;
                case 4:
                    book_format_export = new TOPXBookFormat(filename);
                    break;
                case 5:
                    book_format_export = new REFIBookFormat(filename);
                    break;
                case 6:
                    book_format_export = new SwordBookFormat(filename);
                    break;
                case 7:
                    book_format_export = new BibleAnalyzerBookFormat(filename);
                    break;
                case 8:
                    book_format_export = new MySword4AndroidBookFormat(filename, true);
                    break;
                case 9: // NOT SUPPORTED
                    book_format_export = new STEPBookFormat(filename);
                    break;
                case 10:
                    book_format_export = new SwordSearcherBookFormat(filename);
                    break;

                default:
                    ConsoleUtil.addLog("Sorry, export for this Book format is not yet implemented!");
                    return null;
            }
            return book_format_export;
        }

        public static BookFormat doExportBook(string filename,int filterIdx)
        {
            bool cleaing_required = book_format.CLEANING;
            BookFormat book_format_export = getBookFormat(filename, filterIdx);
            if (book_format_export != null)
            {
                // if the loaded bible requires cleaning, the the target much clean it before export.
                book_format_export.CLEANING = book_format.CLEANING;
                book_format_export.ABBREVIATION = book_format.ABBREVIATION.Replace("'", "''");
                book_format_export.DESCRIPTION = book_format.DESCRIPTION.Replace("'", "''");
                book_format_export.COMMENTS = book_format.COMMENTS.Replace("'", "''");
                if (book_format_export.ABBREVIATION == "BOOK")
                    book_format_export.ABBREVIATION = Path.GetFileNameWithoutExtension(filename).Replace("'", "''");
                if (book_format_export.DESCRIPTION == "Book")
                    book_format_export.DESCRIPTION = Path.GetFileNameWithoutExtension(filename).Replace("'", "''");
            }
            return book_format_export;
        }

        public static bool cleaningRequired(int FilterIdx)
        {
            int[] required = new int[] { 3,4};
            for (int i = 0; i < required.Length; i++)
                if (required[i] == FilterIdx)
                    return true;            
            return false;
        }

		public void SetProcessAsComplete()
		{
			process_complete = true;
		}
		
		
		
		
		public static System.Xml.XmlDocument BookXmlDocument
		{
			get
			{
				return xmldoc;
			}
			set
			{
				xmldoc = value;
			}
		}
		
		public string XML
		{
			get
			{
				return xmldoc.OuterXml;
			}
		}

        public static String PrintXML(String XML)
        {
            String Result = "";

            MemoryStream mStream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(mStream, Encoding.Unicode);
            XmlDocument document = new XmlDocument();

            try
            {
                // Load the XmlDocument with the XML.
                document.LoadXml(XML);

                writer.Formatting = Formatting.Indented;

                // Write the XML into a formatting XmlTextWriter
                document.WriteContentTo(writer);
                writer.Flush();
                mStream.Flush();

                // Have to rewind the MemoryStream in order to read
                // its contents.
                mStream.Position = 0;

                // Read MemoryStream contents into a StreamReader.
                StreamReader sReader = new StreamReader(mStream);

                // Extract the text from the StreamReader.
                String FormattedXML = sReader.ReadToEnd();

                Result = FormattedXML;
            }
            catch (XmlException ex)
            {
                
            }

            mStream.Close();
            writer.Close();

            return Result;
        }        
		
		public string FileName
		{
			get
			{
				return m_filename;
			}
			set
			{
				m_filename = value;
			}
		}
		
		
        public string getNote(string title)
        {
            XmlNode v = xmldoc.SelectSingleNode("//NOTES[@title=\'" + System.Security.SecurityElement.Escape(title) + "\']");
            if (v != null)
                if (v.FirstChild != null)
                    return v.FirstChild.Value;
                return "";
        }
        
        public string getFirstVerse()
        {
            XmlNode v = xmldoc.SelectSingleNode("/XMLBOOK/NOTES[1]");
            if (v != null)
                return v.InnerText;
            else
                return "";
        }


		
		public bool IsComplete
		{
			get
			{
				return process_complete;
			}
		}
		

        public static string postProcessContent(string input, string rtf_header = null)
        {
            string output = input;
            if (GlobalMemory.getInstance().ParseBook)
            {
                if (GlobalMemory.getInstance().ConvertRtfToHtmlBook)
                {
                    output = FormatUtil.convertRtfToHtml(output);
                }
                else if (GlobalMemory.getInstance().autoSetParsingBook && BookFormat.getInstance().CLEANING)
                {
                    // force cleaning rtf to html if set to auto and format requires it.
                    output = FormatUtil.convertRtfToHtml(output);
                }

                if (GlobalMemory.getInstance().ConvertHtmlToRtfBook)
                {
                    output = FormatUtil.convertHtmlToRtf(output);
                }

                // finally strip html tags
                if (GlobalMemory.getInstance().stripHtmlTagsBook)
                {
                    output = FormatUtil.StripHTML(output);
                }
            }
            return output;
        }



        public XmlDocument getXmlDoc()
        {
            return BookXmlDocument;
        }

		public abstract void Load();
		public abstract int PercentComplete {get;}
        public abstract int FilterIndex {get;}

        public abstract void ExportBook(string filename, int idx);
	}
	
}
