using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;

using System;
using System.Collections;

using System.Xml;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace sbrcli
{
	public abstract class CommentaryFormat
	{

        static string filters = null;
        public string ABBREVIATION = "CMT";
        public string DESCRIPTION = "Commentary";
        public string COMMENTS = "Exported from sbrcli (" + GlobalMemory.AUTHOR_WEBSITE + ")";
        public bool CLEANING = false;

        private string m_filename;
		
		private static System.Xml.XmlDocument xmldoc = new System.Xml.XmlDocument();
				
		private bool process_complete = false;

        public string rtf_header = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f1\fnil\fprq2\fcharset161 TITUS Cyberbit Basic;}{\f2\fnil\fprq2\fcharset177 TITUS Cyberbit Basic;}{\f3\fnil\fprq2\fcharset0 TITUS Cyberbit Basic;}{\f4\fnil\fprq2\fcharset136 PMingLiU;}{\f5\fnil\fprq2\fcharset204 Georgia;}{\f6\fnil\fprq2\fcharset238 Georgia;}{\f7\fnil\fprq2\fcharset134 PMingLiU;}{\f8\fnil\fprq2\fcharset222 Cordia New;}{\f9\fnil\fprq2\fcharset129 Gulim;}{\f10\fnil\fprq2\fcharset178 TITUS Cyberbit Basic;}{\f11\fnil\fprq2\fcharset162 Georgia;}{\f12\fnil\fprq2\fcharset163 Times New Roman;}{\f13\fnil\fprq2\fcharset128 MS Mincho;}}{\colortbl ;\red0\green0\blue0;\red0\green0\blue255;\red0\green255\blue255;\red0\green255\blue0;\red255\green0\blue255;\red255\green0\blue0;\red255\green255\blue0;\red255\green255\blue255;\red0\green0\blue128;\red0\green128\blue128;\red0\green128\blue0;\red128\green0\blue128;\red128\green0\blue0;\red128\green128\blue0;\red128\green128\blue128;\red192\green192\blue192;}{\viewkind4\uc1\pard\plain\cf0\fs\par}";
		
		public static CommentaryFormat commentary_format;

		public CommentaryFormat(string filename)
		{
			m_filename = filename;
			process_complete = false;
		}

        public static CommentaryFormat getInstance()
        {
            return commentary_format;
        }

        public static void setInstance(CommentaryFormat mCmtryFormat)
        {
            commentary_format = mCmtryFormat;
        }

        public static string getFormatTypes()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("\t0 - Auto Detect\r\n");  //idx = 0  
            sb.Append("\t1 - Zefania XML Format (*.xml)\r\n");  //idx = 1    
            sb.Append("\t2 - SWORD Commentary Modules (*.zip)\r\n");  //idx = 2       
            sb.Append("\t3 - The Word Commentary Modules (*.cmt.twm)\r\n");  //idx = 3  
            sb.Append("\t5 - e-Sword 9.x and above (CMTX)  (*.cmtx)\r\n");  //idx = 5   
            sb.Append("\t6 - e-Sword HD (CMTI)  (*.cmti)\r\n");  //idx =6
            sb.Append("\t7 - Bible Analyzer Commentary (*.cmt)\r\n");  //idx =7
            sb.Append("\t8 - OnlineBible Commentary (*.exp)\r\n");  //idx =8
            sb.Append("\t9 - MyBible Commentary (*.commentaries.SQLite3, *.commentaries.sqlite3)\r\n");  //idx =9
            sb.Append("\t10 - SwordSearcher/Forge for SwordSearcher (*.ss5cmty, *.txt)\r\n");  //idx =10
            sb.Append("\t11 - MySword for Android (*.cmt.mybible.gz, *.cmt.mybible");  //idx =11
            return sb.ToString();
        }

        public static CommentaryFormat CommentaryFactory(string filename)
        {
            commentary_format = null;            
            if (filename.ToLower().EndsWith(".xml"))
            {
                string xml_str = System.IO.File.ReadAllText(filename).ToLower();
                if (FormatUtil.InStr(xml_str, "<xmlbible"))
                {
                    xml_str = null;
                    commentary_format = new ZefaniaCommentaryFormat(filename);
                }
            }
            else if (filename.ToLower().EndsWith(".zip"))
            {
                commentary_format = new SwordCommentaryFormat(filename);
            }
            else if (System.IO.Directory.Exists(filename))
            {
                //Its a directory
                if (System.IO.Directory.Exists(filename + Path.DirectorySeparatorChar+"mods.d"))
                {
                    commentary_format = new SwordCommentaryFormat(filename);
                }
            }
            else if (filename.ToLower().EndsWith(".cmt.twm"))
            {
                commentary_format = new TheWordCommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".cmtx"))
            {
                commentary_format = new CMTXCommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".cmti"))
            {
                commentary_format = new CMTICommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".cmt"))
            {
                FileStream fs = System.IO.File.OpenRead(filename);                
                int b=fs.ReadByte();
                fs.Close();
                if (b == 0x53) // Checking for S in Sqlite 
                {
                    commentary_format = new BibleAnalyzerCommentaryFormat(filename);
                }                
            }
            else if (filename.ToLower().EndsWith(".cmt.mybible.gz"))
            {
                commentary_format = new MySword4AndroidCommentaryFormat(filename, true);
            }
            else if (filename.ToLower().EndsWith(".exp"))
            {
                commentary_format = new OnlineBibleCommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".cmt.mybible"))
            {
                commentary_format = new MySword4AndroidCommentaryFormat(filename, false);
            }
            else if (filename.ToLower().EndsWith(".commentaries.sqlite3"))
            {
                commentary_format = new MyBibleCommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".ss5cmty"))
            {
                commentary_format = new SwordSearcherCommentaryFormat(filename);
            }
            else if (filename.ToLower().EndsWith(".txt"))
            {
                StreamReader sr = File.OpenText(filename);
                string line = sr.ReadLine();
                sr.Close();
                if (line.StartsWith(";"))
                    commentary_format = new SwordSearcherCommentaryFormat(filename);
                else
                {
                    // any other commentary in text format. nothing yet.
                }
            }
            if (commentary_format != null)
            {
                if (cleaningRequired(commentary_format.FilterIndex))
                {
                    commentary_format.CLEANING = true;
                }
                if (commentary_format.ABBREVIATION == "CMT")
                    commentary_format.ABBREVIATION = Path.GetFileNameWithoutExtension(filename);
                if (commentary_format.DESCRIPTION == "Commentary")
                    commentary_format.DESCRIPTION = Path.GetFileNameWithoutExtension(filename);
            }
            return commentary_format;
        }

        public static CommentaryFormat CommentaryFactory(string filename, int filterIdx)
        {
            if (filterIdx == 0)
                commentary_format = CommentaryFactory(filename);
            else
                commentary_format = getCommentaryFormat(filename, filterIdx);
            return commentary_format;
        }

        public static CommentaryFormat getCommentaryFormat(string filename, int filterIdx)
        {
            CommentaryFormat commentary_format_export = null;
            switch (filterIdx)
            {
                case 1:
                    commentary_format_export = new ZefaniaCommentaryFormat(filename);
                    break;
                case 2:
                    commentary_format_export = new SwordCommentaryFormat(filename);
                    break;
                case 3:
                    commentary_format_export = new TheWordCommentaryFormat(filename);
                    break;
                case 5:
                    commentary_format_export = new CMTXCommentaryFormat(filename);
                    break;
                case 6:
                    commentary_format_export = new CMTICommentaryFormat(filename);
                    break;
                case 7:
                    commentary_format_export = new BibleAnalyzerCommentaryFormat(filename);
                    break;
                case 8:
                    commentary_format_export = new OnlineBibleCommentaryFormat(filename);
                    break;
                case 9:
                    commentary_format_export = new MyBibleCommentaryFormat(filename);
                    break;
                case 10:
                    commentary_format_export = new SwordSearcherCommentaryFormat(filename);
                    break;
                case 11:
                    commentary_format_export = new MySword4AndroidCommentaryFormat(filename, true);
                    break;

                default:
                    if (!GlobalMemory.getInstance().ConsoleMode)
                        ConsoleUtil.addLog("Sorry, export for this Commentary format is not yet implemented!");
                    else
                        ConsoleUtil.addLog("Sorry, export for this Commentary format is not yet implemented!");
                    return null;
            }
            return commentary_format_export;
        }

        public static CommentaryFormat doExportCommentary(string filename, int filterIdx)
        {
            CommentaryFormat commentary_format_export = getCommentaryFormat(filename, filterIdx);
            if (commentary_format_export != null)
            {
                commentary_format_export.CLEANING = commentary_format.CLEANING;
                commentary_format_export.ABBREVIATION = commentary_format.ABBREVIATION.Replace("'", "''");
                commentary_format_export.DESCRIPTION = commentary_format.DESCRIPTION.Replace("'", "''");
                commentary_format_export.COMMENTS = commentary_format.COMMENTS.Replace("'", "''");
                if (commentary_format_export.ABBREVIATION == "CMT")
                    commentary_format_export.ABBREVIATION = Path.GetFileNameWithoutExtension(filename).Replace("'", "''");
                if (commentary_format_export.DESCRIPTION == "Commentary")
                    commentary_format_export.DESCRIPTION = Path.GetFileNameWithoutExtension(filename).Replace("'", "''");
            }
            return commentary_format_export;
        }

        public static string postProcessContent(string input, string rtf_header = null)
        {
            string output = input;
            if (GlobalMemory.getInstance().ParseCommentary)
            {
                if (GlobalMemory.getInstance().ConvertRtfToHtmlCommentary)
                {
                    output = FormatUtil.convertRtfToHtml(output);
                }
                if (GlobalMemory.getInstance().ConvertHtmlToRtfCommentary)
                {
                    output=FormatUtil.convertHtmlToRtf(output);
                }
                // finally strip html tags
                if (GlobalMemory.getInstance().stripHtmlTagsCommentary)
                {
                    output = FormatUtil.StripHTML(output);
                }
            }
            return output;
        }

        public static bool cleaningRequired(int FilterIdx)
        {
            int[] required = new int[] { 3, 4, 5 };
            for (int i = 0; i < required.Length; i++)
                if (required[i] == FilterIdx)
                    return true;            
            return false;
        }

		public void SetProcessAsComplete()
		{
			process_complete = true;
		}		

        public string getFirstCommentary()
        {
            XmlNode v = xmldoc.SelectSingleNode("/XMLBIBLE/BIBLEBOOK[1]/CHAPTER[1]/VERS[1]");
            if (v != null)
                return v.InnerText;
            else
                return "";
        }

		public static System.Xml.XmlDocument CommentaryXmlDocument
		{
			get
			{
				return xmldoc;
			}
			set
			{
				xmldoc = value;
			}
		}
		
		public string XML
		{
			get
			{
				return xmldoc.OuterXml;
			}
		}    
		
		public string FileName
		{
			get
			{
				return m_filename;
			}
			set
			{
				m_filename = value;
			}
		}
		
		public delegate void Clear();
        public delegate int AddItem(string str);

		public bool IsComplete
		{
			get
			{
				return process_complete;
			}
		}

        public static String PrintXML(String XML)
        {
            String Result = "";
            MemoryStream mStream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(mStream, Encoding.UTF8);
            XmlDocument document = new XmlDocument();

            try
            {
                // Load the XmlDocument with the XML.
                document.LoadXml(XML);

                writer.Formatting = Formatting.Indented;

                // Write the XML into a formatting XmlTextWriter
                document.WriteContentTo(writer);
                writer.Flush();
                mStream.Flush();

                // Have to rewind the MemoryStream in order to read
                // its contents.
                mStream.Position = 0;

                // Read MemoryStream contents into a StreamReader.
                StreamReader sReader = new StreamReader(mStream);

                // Extract the text from the StreamReader.
                String FormattedXML = sReader.ReadToEnd();

                Result = FormattedXML;
            }
            catch (XmlException e)
            {
                ConsoleUtil.addLog(e.Message);
                Result = XML;
            }

            mStream.Close();
            writer.Close();

            return Result;
        }

        public XmlDocument getXmlDoc()
        {
            return CommentaryXmlDocument;
        }

		public abstract void Load();
		public abstract int PercentComplete {get;}
        public abstract int FilterIndex {get;}

        public abstract void ExportCommentary(string filename, int idx);
	}
	
}
