﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;
using Microsoft.Win32.SafeHandles;
using Microsoft.Win32;
using static System.Net.Mime.MediaTypeNames;
using System.Reflection;
using Sword;

namespace sbrcli
{
    public class ConsoleApplication
    {
     
        public static Dictionary<string,string> argMap = new Dictionary<string,string>();

        [STAThread]
        public static void Main(string[] args)
        {
            //args = "--format-type bib --input-file kjv-brg.bblx --output-format 18 --output-file kjv --debug true".Split(" ", StringSplitOptions.RemoveEmptyEntries);

            if (args.Length == 1)
            {
                if (args[0] != "--help")
                    ConsoleUtil.addLog("Unknown parameter: " + args[0]+"\r\n");
                printSyntax();
            }

                // make sure there is argument in even count (key/val pair)
            if (args.Length > 0 && args.Length%2==0)
            {
                // Prepare ..

                GlobalMemory.getInstance().ConsoleMode = true;

                for(int i=0;i<args.Length;i+=2) {
                    if(argMap.ContainsKey(args[i].ToLower()))
                        ConsoleUtil.addLog("ERROR: Duplicate parameter "+ args[i].ToLower());
                    else
                        argMap.Add(args[i].ToLower(), args[i+1]);
                }                

                // Parse Input ...

                string format_type = "bib";
                //
                if (argMap.ContainsKey("--format-type"))
                    format_type = argMap["--format-type"].ToLower();

                List<string> allowed_types = new List<string>() {"bib","cmt", "dct", "bok" };
                if(!allowed_types.Contains(format_type))
                {
                    ConsoleUtil.addLog("ERROR: Invalid --format-type value " + argMap["--format-type"]);
                }

                int in_fmt_val = 0;
                if (argMap.ContainsKey("--input-format"))
                {
                    if (!int.TryParse(argMap["--input-format"], out in_fmt_val))
                    {
                        ConsoleUtil.addLog("ERROR: Invalid input format value " + argMap["--input-format"]);
                    }
                }

                string in_File = null;
                if (argMap.ContainsKey("--input-file"))
                {
                    in_File = argMap["--input-file"];
                    if(!File.Exists(in_File))
                    {
                        ConsoleUtil.addLog("ERROR: Input file does not exist.");
                    }
                }
                else
                {
                    ConsoleUtil.addLog("ERROR: --input-file parameter is mandatory.");
                }

                int out_fmt_val = 1;
                if (argMap.ContainsKey("--output-format"))
                {
                    if (!int.TryParse(argMap["--output-format"], out out_fmt_val))
                    {
                        ConsoleUtil.addLog("ERROR: Invalid output format value " + argMap["--output-format"]);
                    }
                }

                string out_File = null; ;
                if (argMap.ContainsKey("--output-file"))
                {
                    out_File = argMap["--output-file"];
                }
                else
                {
                    ConsoleUtil.addLog("ERROR: --output-file parameter is mandatory.");
                }

                /*
                    --metadata-abbr <Abbreviation>
                    --metadata-desc <Description>
                    --metadata-comment <Comment>
                */

                string meta_abbr = format_type.ToUpper();
                if (argMap.ContainsKey("--metadata-abbr"))                
                    meta_abbr = argMap["--metadata-abbr"];
                string meta_desc = format_type.ToUpper();
                if (argMap.ContainsKey("--metadata-desc"))
                    meta_desc = argMap["--metadata-desc"];
                string meta_comment = "Exported from sbrcli (" + GlobalMemory.AUTHOR_WEBSITE + ")";
                if (argMap.ContainsKey("--metadata-comment"))
                    meta_comment = argMap["--metadata-comment"];

                string customParser = null; ;
                GlobalMemory.getInstance().EnableCustomParser = false;
                if (argMap.ContainsKey("--custom-parser"))
                {
                    customParser = argMap["--custom-parser"];
                    if(File.Exists(customParser))
                    {
                        GlobalMemory.getInstance().EnableCustomParser = true;
                        GlobalMemory.getInstance().CustomParserScript = customParser;
                    }
                    else
                        ConsoleUtil.addLog("ERROR: Custom parser script mentioned for --custom-parser does not exist.");
                }

                bool clean = true;
                if (argMap.ContainsKey("--clean"))
                {
                    if (!bool.TryParse(argMap["--clean"], out clean))
                    {
                        ConsoleUtil.addLog("ERROR: --clean parameter value can either be true or false.");
                    }
                }

                if (argMap.ContainsKey("--debug"))
                {
                    if (!bool.TryParse(argMap["--debug"], out GlobalMemory.getInstance().dev_mode))
                    {
                        ConsoleUtil.addLog("ERROR: --debug parameter value can either be true or false.");
                    }
                }

                ConsoleUtil.logFile = "console";
                if (argMap.ContainsKey("--log"))
                    ConsoleUtil.logFile = argMap["--log"];                    ;

                // set preferences

                switch (format_type)
                {
                    case "cmt":
                        if (argMap.ContainsKey("--parse"))
                            bool.TryParse(argMap["--parse"], out GlobalMemory.getInstance().ParseCommentary);
                        if (argMap.ContainsKey("--autoset-parsing"))
                            bool.TryParse(argMap["--autoset-parsing"], out GlobalMemory.getInstance().autoSetParsingCommentary);
                        if (argMap.ContainsKey("--convert-rtf-to-html"))
                            bool.TryParse(argMap["--convert-rtf-to-html"], out GlobalMemory.getInstance().ConvertRtfToHtmlCommentary);
                        if (argMap.ContainsKey("--convert-html-to-rtf"))
                            bool.TryParse(argMap["--convert-html-to-rtf"], out GlobalMemory.getInstance().ConvertHtmlToRtfCommentary);
                        if (argMap.ContainsKey("--strip-html-tags"))
                            bool.TryParse(argMap["--strip-html-tags"], out GlobalMemory.getInstance().stripHtmlTagsCommentary);
                        break;

                    case "dct":
                        if (argMap.ContainsKey("--parse"))
                            bool.TryParse(argMap["--parse"], out GlobalMemory.getInstance().ParseDictionary);
                        if (argMap.ContainsKey("--autoset-parsing"))
                            bool.TryParse(argMap["--autoset-parsing"], out GlobalMemory.getInstance().autoSetParsingDictionary);
                        if (argMap.ContainsKey("--convert-rtf-to-html"))
                            bool.TryParse(argMap["--convert-rtf-to-html"], out GlobalMemory.getInstance().ConvertRtfToHtmlDictionary);
                        if (argMap.ContainsKey("--convert-html-to-rtf"))
                            bool.TryParse(argMap["--convert-html-to-rtf"], out GlobalMemory.getInstance().ConvertHtmlToRtfDictionary);
                        if (argMap.ContainsKey("--strip-html-tags"))
                            bool.TryParse(argMap["--strip-html-tags"], out GlobalMemory.getInstance().stripHtmlTagsDictionary);
                        break;

                    case "bok":
                        if (argMap.ContainsKey("--parse"))
                            bool.TryParse(argMap["--parse"], out GlobalMemory.getInstance().ParseBook);
                        if (argMap.ContainsKey("--autoset-parsing"))
                            bool.TryParse(argMap["--autoset-parsing"], out GlobalMemory.getInstance().autoSetParsingBook);
                        if (argMap.ContainsKey("--convert-rtf-to-html"))
                            bool.TryParse(argMap["--convert-rtf-to-html"], out GlobalMemory.getInstance().ConvertRtfToHtmlBook);
                        if (argMap.ContainsKey("--convert-html-to-rtf"))
                            bool.TryParse(argMap["--convert-html-to-rtf"], out GlobalMemory.getInstance().ConvertHtmlToRtfBook);
                        if (argMap.ContainsKey("--strip-html-tags"))
                            bool.TryParse(argMap["--strip-html-tags"], out GlobalMemory.getInstance().stripHtmlTagsBook);
                        break;

                    case "bib":
                    default:
                        if (argMap.ContainsKey("--parse"))
                            bool.TryParse(argMap["--parse"], out GlobalMemory.getInstance().ParseBible);
                        if (argMap.ContainsKey("--autoset-parsing"))
                            bool.TryParse(argMap["--autoset-parsing"], out GlobalMemory.getInstance().autoSetParsingBible);
                        if (argMap.ContainsKey("--convert-rtf-to-html"))
                            bool.TryParse(argMap["--convert-rtf-to-html"], out GlobalMemory.getInstance().ConvertRtfToHtmlBible);
                        if (argMap.ContainsKey("--convert-html-to-rtf"))
                            bool.TryParse(argMap["--convert-html-to-rtf"], out GlobalMemory.getInstance().ConvertHtmlToRtfBible);
                        if (argMap.ContainsKey("--strip-html-tags"))
                            bool.TryParse(argMap["--strip-html-tags"], out GlobalMemory.getInstance().stripHtmlTagsBible);
                        if (argMap.ContainsKey("--add-red-text"))
                            bool.TryParse(argMap["--add-red-text"], out GlobalMemory.getInstance().addRedText);
                        if (argMap.ContainsKey("--sword-versification"))
                            GlobalMemory.getInstance().Versification = argMap["--sword-versification"].ToLower();
                        break;
                }

                // detect sword engine
                ConsoleUtil.checkAndSetSwordLibrary();

                // initialize format util.
                FormatUtil.Init();
                // Execute ...

                if (ConsoleUtil.msg.Count == 0)                
                    ConsoleUtil.Convert(format_type, in_fmt_val, in_File, out_fmt_val, out_File, clean, meta_abbr, meta_desc, meta_comment);

                switch (ConsoleUtil.logFile)
                {
                    case "console":
                        ConsoleUtil.displayLog();
                        break;
                    case "nolog":
                        // silent
                        break;
                    default:
                        ConsoleUtil.writeLog(ConsoleUtil.logFile);
                        break;
                }
            }
            else
            {
                //ABBR
                //DESC
                //COMMENT
                printSyntax();
            }
        }

        public static void printSyntax()
        {
            string syntax = @"
@TITLE@ (@PRODUCT@) v@VERSION@
@DESCRIPTION@
License: @LICENSE@
Website: @TRADEMARK@

Syntax:
    sbrcli [Options]

Below are the list of supported options:

--format-type [Format-Type]
Format type defines if the conversion is a bible, commentary, dictionary or a book. The following values are accepted. This is optional. If not specified, the default value is BIB.
    BIB - Bible
    CMT - Commentary
    DCT - Dictionary
    BOK - Book

--input-format <Format-Value>
Refer Format Values section.

--input-file <Input-File>
Absolute or relative input file path.

--output-format <Format-Value>
Refer Format Values section.

FORMAT VALUES:
The following format values are supported for --input-format or --output-format. This is optional. If not specified, the default value for --input-format is auto detect based on file extension and the default value for --output-format is 1 (Zefania XML).
Format values for bible (format type BIB)
@BIB_FORMAT@
Format values for commentary (format type CMT)
@CMT_FORMAT@
Format values for dictionary (format type DCT)
@DCT_FORMAT@
Format values for book (format type BOK)
@BOK_FORMAT@

--output-file <OutputFile>
Absolute or relative output file path.

--metadata-abbr <Abbreviation>
Specify abbreviation on output file if the format supports. Default is the abbreviation found in input file.

--metadata-desc <Description>
Specify description on output file if the format supports. Default is the description found in input file.

--metadata-comment <Comment>
Specify comment/note/information on output file if the format supports. Default is 'Exported from sbrcli application'.

--custom-parser [Custom parser script]
Provide a custom parser as a .rxp plugin or batch/shell script. For batch/shell scripts, verse input will in VERSE_IN environment variable and the script output in console will be catprued as parsed output.

--clean [true or false]
Parses and clean RTF tags for Bibles and Books while exporting. Value can be either true or false. This is optional and default is true.

--log [File Path, console or nolog] 
Log file path. This is optional and default is console.

PREFERENCES:
The following options are optional and common for all format types. The default is true.
--parse [true or false]
Enables parsing of input text. If set to false, the output will have raw data as provided in input.

--autoset-parsing [true or false]
Auto parsing allows the program to automatically set parsing options (rtf/html/strip tag etc) based on input type. This option will be ignored if --parse is set to false.

--convert-rtf-to-html [true or false]
Converts RTF to HTML in output. This option will be ignored if either --parse or --autoset-parsing is set to false.

--convert-html-to-rtf [true or false]
Converts HTML to RTF in output. This option will be ignored if either --parse or --autoset-parsing is set to false.

--strip-html-tags [true or false]
Strips HTML tags from output. This option will be ignored if either --parse or --autoset-parsing is set to false.

--sword-versification [Versification]
The following options for sword versification is valid on Windows. On Linux platforms, they are valid only if sword engine (libsword) is installed. If the sword engine is not installed, cross-connect library will be used with no apographa and only KJV versification support. The below options are optional and only available for bible format type. i.e., BIB. If no versification is provided, the default KJVA versification will be used. The below mentioned versifications are supported.
    KJV         - King James Version
    CALVIN      - French Bible d'Olivétan and followers, 1560+ Bible de Genève, 1707-1855, etc. 
    CATHOLIC    - Catholic Bibles with 10 chapters in Esther
    CATHOLIC2   - Catholic Bibles with 16 chapters in Esther
    DARBYFR     - French Bibles by J.N. Darby
    GERMAN      - Identical to Luther, but without the deuterocanonical books and with Vulgate/KJV book order.
    KJVA        - KJV + Apocrypha
    LENINGRAD   - Leningrad Codex, the book order is different
    LUTHER      - Luther
    LXX         - Greek LXX & translations of the LXX
    MT          - Masoretic Text for Hebrew Bibles
    NRSV        - New Revised Standard Version
    NRSVA       - NRSV + Apocrypha
    ORTHODOX    - LXX, but different book order with fewer books
    SEGOND      - French Bibles L. Segond 1880, 1910, 1978, 2002 (NBS), 2007 (SG21)
    SYNODAL     - Russian Synodal Bible and related Bibles
    SYNODALPROT - The ParaText Synodal Protestant used by IBT for Bibles having protocanonical books only
    VULG        - Vulgate
@SWORDDETECT@

--add-red-text [true or false]
Adds red text styling for word of Jesus Christ.

--debug [true or false]
Prints debug messages to log.

E.g.,
    sbrcli --format-type bib --input-file kjv.ont --output-file kjv.bbl

";
            ConsoleUtil.checkAndSetSwordLibrary();

            syntax = syntax.Replace("@TITLE@", AssemblyInfo.Title);
            syntax = syntax.Replace("@PRODUCT@", AssemblyInfo.Product);
            syntax = syntax.Replace("@VERSION@", AssemblyInfo.VersionFull);
            syntax = syntax.Replace("@LICENSE@", AssemblyInfo.Copyright);
            syntax = syntax.Replace("@DESCRIPTION@", AssemblyInfo.Description);
            syntax = syntax.Replace("@TRADEMARK@", AssemblyInfo.Trademark);

            syntax = syntax.Replace("@BIB_FORMAT@", BibleFormat.getFormatTypes());
            syntax = syntax.Replace("@CMT_FORMAT@", CommentaryFormat.getFormatTypes());
            syntax = syntax.Replace("@DCT_FORMAT@", DictionaryFormat.getFormatTypes());
            syntax = syntax.Replace("@BOK_FORMAT@", BookFormat.getFormatTypes());

            if(GlobalMemory.getInstance().use_libsword)
                syntax = syntax.Replace("@SWORDDETECT@", "The Sword Engine library is detected on this system.");
            else
                syntax = syntax.Replace("@SWORDDETECT@", "The Sword Engine library is not detected. Cross-connect library will be used.");

            Console.WriteLine(syntax);
        }

    }
}
