﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Xml;

namespace sbrcli
{
    internal class CustomParser
    { 
        System.Xml.XmlDocument xmlDoc;
        int pluginType;

        int mMax = 100;
        int mCurr = 0;
        public static void doParse(string format_type)
        {

            if (GlobalMemory.getInstance().EnableCustomParser)
            {
                ConsoleUtil.CLIDebug("Processing Custom Parser ..");
                string custom_parser = GlobalMemory.getInstance().CustomParserScript;                
                Random rand = new Random(DateTime.Now.TimeOfDay.Milliseconds);

                XmlNodeList leaves = null;
                if (format_type.ToLower()=="bib")
                    leaves = BibleFormat.getInstance().getXmlDoc().SelectNodes("//VERS");
                else if (format_type.ToLower() == "cmt")
                    leaves = CommentaryFormat.getInstance().getXmlDoc().SelectNodes("//VERS");
                else if (format_type.ToLower() == "bok")
                    leaves = BookFormat.getInstance().getXmlDoc().SelectNodes("//NOTES");
                else if (format_type.ToLower() == "dct")
                    leaves = DictionaryFormat.getInstance().getXmlDoc().SelectNodes("///item");
                if (leaves == null)
                {
                    // error. this shouldn't happen
                    ConsoleUtil.CLIDebug("There is no data available for parsing. Input possibly wrong format.");
                    ConsoleUtil.addLog("No data found in input");
                    return;

                }
                ConsoleUtil.CLIDebug("Total content to process:" + leaves.Count.ToString());
                //
                int count = 0;

                string outData = null;
                string inData = null;
                string[] lines = null;

                List<string> pList = new List<string>();

                if (custom_parser.EndsWith(".rxp"))
                {
                    ConsoleUtil.CLIDebug("Custom Parser is a regular expression plugin.");
                    lines = File.ReadAllLines(custom_parser);
                    foreach (string line in lines)
                    {
                        if (line.Trim() != "" && !line.Trim().StartsWith("#"))
                            pList.Add(line);
                    }


                    if (pList.Count > 0)
                    {
                        foreach (XmlNode leaf in leaves)
                        {
                            for (int i = 0; i < pList.Count; i += 2)
                            {
                                inData = leaf.InnerText;
                                outData = Regex.Replace(inData, pList[i], pList[i + 1]);
                                try
                                {
                                    outData = outData.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;").Replace("\"", "&quot;").Replace("'", "&apos;");
                                    leaf.InnerXml = outData;
                                }
                                catch (Exception)
                                {
                                    leaf.InnerText = outData;
                                }
                            }
                            count++;
                        }
                    }
                }
                else
                {
                    ConsoleUtil.CLIDebug("Custom Parser is a script.");
                    int idx = 0;
                    int cnt = leaves.Count;
                    foreach (XmlNode leaf in leaves)
                    {
                        if (idx % 1000 == 0)
                            ConsoleUtil.CLIDebug(idx + "/" + cnt + " completed.");
                        idx++;

                        Process p = new Process();
                        p.StartInfo.UseShellExecute = false;
                        p.StartInfo.RedirectStandardOutput = true;
                        p.StartInfo.WorkingDirectory = AppContext.BaseDirectory;
                        p.StartInfo.FileName = custom_parser;
                        Environment.SetEnvironmentVariable("VERSE_IN", leaf.InnerText);
                        //foreach(var env in p.StartInfo.EnvironmentVariables.Keys)
                        //Console.WriteLine(env+" = "+p.StartInfo.EnvironmentVariables[env.ToString()]);
                        p.StartInfo.CreateNoWindow = true;
                        p.Start();
                        outData = p.StandardOutput.ReadToEnd();
                        p.WaitForExit();
                        
                        try
                        {
                            outData = outData.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;").Replace("\"", "&quot;").Replace("'", "&apos;");
                            leaf.InnerXml = outData;
                        }
                        catch (Exception)
                        {
                            leaf.InnerText = outData;
                        }
                        //
                    }
                }
                ConsoleUtil.CLIDebug("Custom Parser execution completed.");
            }
        }
    }
}
